@@ -1,5 +1,5 @@
 # isort:skip_file
-__version__ = "0.1.1"
+__version__ = "0.1.2"
 
 
 from treeo.api import (
@@ -13,6 +13,7 @@ from treeo.api import (
     to_string,
     compact,
     mutable,
+    toplevel_mutable,
 )
 from treeo.mixins import (
     Copy,
@@ -29,7 +30,7 @@ from treeo.mixins import (
     Immutable,
     ImmutableTree,
 )
-from treeo.tree import FlattenMode, FieldInfo, TreeMeta, Tree, copy, apply
+from treeo.tree import FlattenMode, FieldInfo, TreeMeta, Tree, copy, apply, make_mutable
 from treeo.types import FieldMetadata, Nothing, NOTHING, Missing, MISSING, Hashable
 from treeo.utils import OpaquePredicate, ArrayLike, Opaque, field, node, static
 
@@ -72,8 +73,10 @@ __all__ = [
     "map",
     "merge",
     "mutable",
+    "toplevel_mutable",
     "node",
     "static",
     "to_dict",
     "to_string",
+    "make_mutable",
 ]
