@@ -24,6 +24,7 @@ RICH_WARNING_COUNT = 0
 
 A = tp.TypeVar("A")
 B = tp.TypeVar("B")
+C = tp.TypeVar("C", bound=tp.Callable)
 T = tp.TypeVar("T", bound="Tree")
 Filter = tp.Union[
     tp.Type[tp.Any],
@@ -232,7 +233,7 @@ def to_dict(
 
 
 def _remove_field_info_from_metadata(obj: Tree):
-    with tree_m._make_mutable_single(obj):
+    with tree_m._make_mutable_toplevel(obj):
         obj._field_metadata = jax.tree_map(
             lambda x: x.value if isinstance(x, FieldInfo) else x,
             obj._field_metadata,
@@ -482,7 +483,11 @@ def compact(f):
     return wrapper
 
 
-def mutable(f: tp.Callable[..., A]) -> tp.Callable[..., tp.Tuple[A, tp.Any]]:
+def mutable(
+    f: tp.Callable[..., A],
+    *,
+    toplevel_only: bool = False,
+) -> tp.Callable[..., tp.Tuple[A, tp.Any]]:
     """
     A decorator that transforms a stateful function `f` that receives an Tree
     instance as a its first argument into a function that returns a tuple of the result and a Tree
@@ -509,8 +514,12 @@ def mutable(f: tp.Callable[..., A]) -> tp.Callable[..., tp.Tuple[A, tp.Any]]:
     assert y == 1
     ```
 
+    **Note**: Any `Tree`s that are found in the output of `f` are set to being
+    immutable.
+
     Arguments:
         f: The function to be transformed.
+        toplevel_only: If `True`, only the top-level object is made mutable.
 
     Returns:
         A function that returns a tuple of the result and a Tree with the new state.
@@ -520,14 +529,91 @@ def mutable(f: tp.Callable[..., A]) -> tp.Callable[..., tp.Tuple[A, tp.Any]]:
     def wrapper(tree, *args, **kwargs) -> tp.Tuple[A, tp.Any]:
         tree = tree_m.copy(tree)
 
-        with tree_m._make_mutable(tree):
+        with tree_m.make_mutable(tree, toplevel_only=toplevel_only):
             output = f(tree, *args, **kwargs)
 
+        def _make_output_immutable(a: Tree):
+            # update __dict__ instead to avoid error during __setattr__
+            tree_m._set_mutable(a, None)
+
+        tree_m.apply(_make_output_immutable, output, inplace=True)
+
         return output, tree
 
     return wrapper
 
 
+def toplevel_mutable(f: C) -> C:
+    """
+    A decorator that transforms a stateful function `f` that receives an Tree
+    instance as a its first argument into a mutable function. It differs from `mutable`
+    in the following ways:
+
+    * It always applies mutability to the top-level object only.
+    * `f` is expected to return the new state either as the only output or
+        as the last element of a tuple.
+
+    Example:
+
+    ```python
+    @dataclass
+    class Child(to.Tree, to.Immutable):
+        n: int = to.node()
+
+    @dataclass
+    def Parent(to.Tree, to.Immutable):
+        child: Child
+
+        @to.toplevel_mutable
+        def update(self) -> "Parent":
+            # self is currently mutable
+            self.child = self.child.replace(n=self.child.n + 1) # but child is immutable (so we use replace)
+
+            return self
+
+    tree = Parent(child=Child(n=4))
+    tree = tree.update()
+    ```
+
+    This behaviour is useful when the top-level tree mostly manipulates sub-trees that have well-defined
+    immutable APIs, avoids explicitly run `replace` to propagate updates to the sub-trees and makes
+    management of the top-level tree easier.
+
+    **Note**: Any `Tree`s that are found in the output of `f` are set to being
+    immutable, however the element is the to the same immutablity status as the
+    input tree if they have the same type.
+
+    Arguments:
+        f: The function to be transformed.
+
+    Returns:
+        A function with top-level mutability.
+    """
+
+    @functools.wraps(f)
+    def wrapper(tree: tree_m.Tree, *args, **kwargs):
+        if not isinstance(tree, tree_m.Tree):
+            raise TypeError(f"Expected 'Tree' type, got '{type(tree).__name__}'")
+
+        output, _ = mutable(f, toplevel_only=True)(tree, *args, **kwargs)
+
+        if isinstance(output, tuple):
+            *ys, last = output
+        else:
+            ys = ()
+            last = output
+
+        if type(last) is type(tree):
+            tree_m._set_mutable(last, tree._mutable)
+
+        if isinstance(output, tuple):
+            return (*ys, last)
+        else:
+            return last
+
+    return wrapper
+
+
 # ---------------------------------------------------------------
 # utils
 # ---------------------------------------------------------------
