@@ -276,7 +276,7 @@ class Compact:
 
             value = initializer()
 
-            with tree_m._make_mutable_single(self):
+            with tree_m._make_mutable_toplevel(self):
                 setattr(self, field_name, value)
 
         return value
@@ -373,7 +373,7 @@ class Immutable:
     which let you modify the state by creating a new objects.
     """
 
-    _mutable: bool
+    _mutable: tp.Optional[tree_m._MutableState]
 
     def replace(self: A, **kwargs) -> A:
         """
@@ -401,16 +401,18 @@ class Immutable:
         """
         tree = tree_m.copy(self)
 
-        with tree_m._make_mutable_single(tree):
+        with tree_m._make_mutable_toplevel(tree):
             for key, value in kwargs.items():
                 setattr(tree, key, value)
 
-        return tree
+        # return a copy to potentially update metadata
+        return tree_m.copy(tree)
 
     def mutable(
         self: A,
         *args,
         method: tp.Union[str, tp.Callable] = "__call__",
+        toplevel_only: bool = False,
         **kwargs,
     ) -> tp.Tuple[tp.Any, A]:
         """
@@ -442,6 +444,7 @@ class Immutable:
             *args: The positional arguments to pass to the method.
             method: The method to call, can be a string with the method name,
                 a bounded method, or a function that takes the object as first argument.
+            toplevel_only: If `True` only the top level object will be made mutable.
             **kwargs: The keyword arguments to pass to the method.
 
         Returns:
@@ -449,15 +452,60 @@ class Immutable:
             updated tree.
         """
 
-        unbounded_method: tp.Callable = (
-            getattr(self.__class__, method)
-            if isinstance(method, str)
-            else method.__func__
-            if inspect.ismethod(method)
-            else method
+        unbounded_method = utils._get_unbound_method(self, method)
+        return api.mutable(unbounded_method, toplevel_only=toplevel_only)(
+            self, *args, **kwargs
         )
 
-        return api.mutable(unbounded_method)(self, *args, **kwargs)
+    def toplevel_mutable(
+        self, *args, method: tp.Union[str, tp.Callable] = "__call__", **kwargs
+    ) -> tp.Any:
+        """
+        Calls a method that contains stateful/mutable operations in
+        an immutable fashion. It differs from `mutable` in the following ways:
+
+        * Mutability is granted only for the current top-level Tree, sub-trees are not affected.
+        * `method` is expected to return the new state (usually self) either as the only output or
+            as the last element of a tuple.
+
+        Note that since the original object is not modified, `Immutable` instance remain in the end immutable.
+
+        Example:
+
+        ```python
+        @dataclass
+        class Child(to.Tree, to.Immutable):
+            n: int = to.node()
+
+        @dataclass
+        def Parent(to.Tree, to.Immutable):
+            child: Child
+
+            def update(self) -> "Parent":
+                # self is currently mutable
+                self.child = self.child.replace(n=self.child.n + 1) # but child is immutable (so we use replace)
+
+                return self
+
+        tree = Parent(child=Child(n=4))
+        tree = tree.toplevel_mutable(method="update")
+        ```
+
+        This behaviour is useful when the top-level tree mostly manipulates sub-trees that have well-defined
+        immutable APIs, avoids explicitly run `replace` to propagate updates to the sub-trees and makes
+        management of the top-level tree easier.
+
+        Arguments:
+            *args: The positional arguments to pass to the method.
+            method: The method to call, can be a string with the method name,
+                a bounded method, or a function that takes the object as first argument.
+            **kwargs: The keyword arguments to pass to the method.
+
+        Returns:
+            Same outputs as `method`.
+        """
+        unbounded_method = utils._get_unbound_method(self, method)
+        return api.toplevel_mutable(unbounded_method)(self, *args, **kwargs)
 
 
 # define __setattr__ outside of class so linters still detect it unknown attribute assignments
